@@ -19,21 +19,22 @@
 #include <iomanip>
 #include <iostream>
 #include <list>
-#include <tuple>
-#include <string>
 #include <sstream>
+#include <string>
+#include <tuple>
 #include <utility>
 
-#include "libshaderc_util/compiler.h"
-#include "libshaderc_util/io.h"
-#include "libshaderc_util/string_piece.h"
-#include "shaderc/shaderc.h"
-#include "spirv-tools/libspirv.h"
-
 #include "file.h"
 #include "file_compiler.h"
+#include "libshaderc_util/args.h"
+#include "libshaderc_util/compiler.h"
+#include "libshaderc_util/io_shaderc.h"
+#include "libshaderc_util/string_piece.h"
 #include "resource_parse.h"
 #include "shader_stage.h"
+#include "shaderc/env.h"
+#include "shaderc/shaderc.h"
+#include "spirv-tools/libspirv.h"
 
 using shaderc_util::string_piece;
 
@@ -60,15 +61,21 @@ Options:
                     Automatically assign locations to uniform variables that
                     don't have an explicit 'location' layout in the shader
                     source.
+  -fauto-combined-image-sampler
+                    Removes sampler variables and converts existing textures
+                    to combined image-samplers.
   -fentry-point=<name>
                     Specify the entry point name for HLSL compilation, for
                     all subsequent source files.  Default is "main".
+  -fhlsl-16bit-types
+                    Enable 16-bit type support for HLSL.
   -fhlsl_functionality1, -fhlsl-functionality1
                     Enable extension SPV_GOOGLE_hlsl_functionality1 for HLSL
                     compilation.
   -fhlsl-iomap      Use HLSL IO mappings for bindings.
   -fhlsl-offsets    Use HLSL offset rules for packing members of blocks.
                     Affects only GLSL.  HLSL rules are always used for HLSL.
+  -finvert-y        Invert position.Y output in vertex shader.
   -flimit=<settings>
                     Specify resource limits. Each limit is specified by a limit
                     name followed by an integer value.  Tokens should be
@@ -76,6 +83,10 @@ Options:
                     several times, only the last setting takes effect.
   -flimit-file <file>
                     Set limits as specified in the given file.
+  -fnan-clamp       Generate code for max and min builtins so that, when given
+                    a NaN operand, the other operand is returned. Similarly,
+                    the clamp builtin will favour the non-NaN operands, as if
+                    clamp were implemented as a composition of max and min.
   -fresource-set-binding [stage] <reg0> <set0> <binding0>
                         [<reg1> <set1> <binding1>...]
                     Explicitly sets the descriptor set and binding for
@@ -118,13 +129,15 @@ Options:
                     Valid stages are vertex, vert, fragment, frag, tesscontrol,
                     tesc, tesseval, tese, geometry, geom, compute, and comp.
   -g                Generate source-level debug information.
-                    Currently this option has no effect.
+  -h                Display available options.
   --help            Display available options.
   -I <value>        Add directory to include search path.
   -mfmt=<format>    Output SPIR-V binary code using the selected format. This
                     option may be specified only when the compilation output is
-                    in SPIR-V binary code form. Available options include bin, c
-                    and num. By default the binary output format is bin.
+                    in SPIR-V binary code form. Available options are:
+                      bin   - SPIR-V binary words.  This is the default.
+                      c     - Binary words as C initializer list of 32-bit ints
+                      num   - List of comma-separated 32-bit hex integers
   -M                Generate make dependencies. Implies -E and -w.
   -MM               An alias for -M.
   -MD               Generate make dependencies and compile.
@@ -139,7 +152,7 @@ Options:
   -std=<value>      Version and profile for GLSL input files. Possible values
                     are concatenations of version and profile, e.g. 310es,
                     450core, etc.  Ignored for HLSL files.
-  -S                Only run preprocess and compilation steps.
+  -S                Emit SPIR-V assembly instead of binary.
   --show-limits     Display available limit names and their default values.
   --target-env=<environment>
                     Set the target client environment, and the semantics
@@ -147,9 +160,21 @@ Options:
                     the client version.  Values are:
                         vulkan1.0       # The default
                         vulkan1.1
+                        vulkan1.2
+                        vulkan1.3
                         vulkan          # Same as vulkan1.0
                         opengl4.5
                         opengl          # Same as opengl4.5
+  --target-spv=<spirv-version>
+                    Set the SPIR-V version to be used for the generated SPIR-V
+                    module.  The default is the highest version of SPIR-V
+                    required to be supported for the target environment.
+                    For example, default for vulkan1.0 is spv1.0, and
+                    the default for vulkan1.1 is spv1.3,
+                    the default for vulkan1.2 is spv1.5.
+                    the default for vulkan1.3 is spv1.6.
+                    Values are:
+                        spv1.0, spv1.1, spv1.2, spv1.3, spv1.4, spv1.5, spv1.6
   --version         Display compiler version information.
   -w                Suppresses all warning messages.
   -Werror           Treat all warnings as errors.
@@ -160,29 +185,6 @@ Options:
 )";
 }
 
-// Gets the option argument for the option at *index in argv in a way consistent
-// with clang/gcc. On success, returns true and writes the parsed argument into
-// *option_argument. Returns false if any errors occur. After calling this
-// function, *index will be the index of the last command line argument consumed.
-bool GetOptionArgument(int argc, char** argv, int* index,
-                       const std::string& option,
-                       string_piece* option_argument) {
-  const string_piece arg = argv[*index];
-  assert(arg.starts_with(option));
-  if (arg.size() != option.size()) {
-    *option_argument = arg.substr(option.size());
-    return true;
-  } else {
-    if (option.back() == '=') {
-      *option_argument = "";
-      return true;
-    }
-    if (++(*index) >= argc) return false;
-    *option_argument = argv[*index];
-    return true;
-  }
-}
-
 // Sets resource limits according to the given string. The string
 // should be formated as required for ParseResourceSettings.
 // Returns true on success.  Otherwise returns false and sets err
@@ -203,31 +205,6 @@ const char kBuildVersion[] =
 #include "build-version.inc"
     ;
 
-
-// Parses the given string as a number of the specified type.  Returns true
-// if parsing succeeded, and stores the parsed value via |value|.
-// (I've worked out the general case for this in
-// SPIRV-Tools source/util/parse_number.h. -- dneto)
-bool ParseUint32(const std::string& str, uint32_t* value) {
-  std::istringstream iss(str);
-
-  iss >> std::setbase(0);
-  iss >> *value;
-
-  // We should have read something.
-  bool ok = !str.empty() && !iss.bad();
-  // It should have been all the text.
-  ok = ok && iss.eof();
-  // It should have been in range.
-  ok = ok && !iss.fail();
-
-  // Work around a bugs in various C++ standard libraries.
-  // Count any negative number as an error, including "-0".
-  ok = ok && (str[0] != '-');
-
-  return ok;
-}
-
 // Gets an optional stage name followed by required offset argument.  Returns
 // false and emits a message to *errs if any errors occur.  After calling this
 // function, *index will be the index of the last command line argument
@@ -253,7 +230,7 @@ bool GetOptionalStageThenOffsetArgument(const shaderc_util::string_piece option,
       return false;
     }
   }
-  if (!ParseUint32(argv[argi + 1], offset)) {
+  if (!shaderc_util::ParseUint32(argv[argi + 1], offset)) {
     *errs << "glslc: error: invalid offset value " << argv[argi + 1] << " for "
           << option << std::endl;
     return false;
@@ -295,7 +272,7 @@ int main(int argc, char** argv) {
 
   for (int i = 1; i < argc; ++i) {
     const string_piece arg = argv[i];
-    if (arg == "--help") {
+    if (arg == "--help" || arg == "-h") {
       ::PrintHelp(&std::cout);
       return 0;
     } else if (arg == "--show-limits") {
@@ -319,7 +296,7 @@ int main(int argc, char** argv) {
       return 0;
     } else if (arg.starts_with("-o")) {
       string_piece file_name;
-      if (!GetOptionArgument(argc, argv, &i, "-o", &file_name)) {
+      if (!shaderc_util::GetOptionArgument(argc, argv, &i, "-o", &file_name)) {
         std::cerr
             << "glslc: error: argument to '-o' is missing (expected 1 value)"
             << std::endl;
@@ -336,14 +313,23 @@ int main(int argc, char** argv) {
       }
     } else if (arg == "-fauto-bind-uniforms") {
       compiler.options().SetAutoBindUniforms(true);
+    } else if (arg == "-fauto-combined-image-sampler") {
+      compiler.options().SetAutoSampledTextures(true);
     } else if (arg == "-fauto-map-locations") {
       compiler.options().SetAutoMapLocations(true);
     } else if (arg == "-fhlsl-iomap") {
       compiler.options().SetHlslIoMapping(true);
     } else if (arg == "-fhlsl-offsets") {
       compiler.options().SetHlslOffsets(true);
-    } else if (arg == "-fhlsl_functionality1" || arg == "-fhlsl-functionality1") {
+    } else if (arg == "-fhlsl_functionality1" ||
+               arg == "-fhlsl-functionality1") {
       compiler.options().SetHlslFunctionality1(true);
+    } else if (arg == "-fhlsl-16bit-types") {
+      compiler.options().SetHlsl16BitTypes(true);
+    } else if (arg == "-finvert-y") {
+      compiler.options().SetInvertY(true);
+    } else if (arg == "-fnan-clamp") {
+      compiler.options().SetNanClamp(true);
     } else if (((u_kind = shaderc_uniform_kind_image),
                 (arg == "-fimage-binding-base")) ||
                ((u_kind = shaderc_uniform_kind_texture),
@@ -378,13 +364,13 @@ int main(int argc, char** argv) {
              argv[i + 3][0] != '-') {
         seen_triple = true;
         uint32_t set = 0;
-        if (!ParseUint32(argv[i + 2], &set)) {
+        if (!shaderc_util::ParseUint32(argv[i + 2], &set)) {
           std::cerr << "glslc: error: Invalid set number: " << argv[i + 2]
                     << std::endl;
           return 1;
         }
         uint32_t binding = 0;
-        if (!ParseUint32(argv[i + 3], &binding)) {
+        if (!shaderc_util::ParseUint32(argv[i + 3], &binding)) {
           std::cerr << "glslc: error: Invalid binding number: " << argv[i + 3]
                     << std::endl;
           return 1;
@@ -412,7 +398,8 @@ int main(int argc, char** argv) {
     } else if (arg.starts_with("-flimit-file")) {
       std::string err;
       string_piece limits_file;
-      if (!GetOptionArgument(argc, argv, &i, "-flimit-file", &limits_file)) {
+      if (!shaderc_util::GetOptionArgument(argc, argv, &i, "-flimit-file",
+                                           &limits_file)) {
         std::cerr << "glslc: error: argument to '-flimit-file' is missing"
                   << std::endl;
         return 1;
@@ -454,6 +441,12 @@ int main(int argc, char** argv) {
       } else if (target_env_str == "vulkan1.1") {
         target_env = shaderc_target_env_vulkan;
         version = shaderc_env_version_vulkan_1_1;
+      } else if (target_env_str == "vulkan1.2") {
+        target_env = shaderc_target_env_vulkan;
+        version = shaderc_env_version_vulkan_1_2;
+      } else if (target_env_str == "vulkan1.3") {
+        target_env = shaderc_target_env_vulkan;
+        version = shaderc_env_version_vulkan_1_3;
       } else if (target_env_str == "opengl") {
         target_env = shaderc_target_env_opengl;
       } else if (target_env_str == "opengl4.5") {
@@ -461,6 +454,9 @@ int main(int argc, char** argv) {
         version = shaderc_env_version_opengl_4_5;
       } else if (target_env_str == "opengl_compat") {
         target_env = shaderc_target_env_opengl_compat;
+        std::cerr << "glslc: error: opengl_compat is no longer supported"
+                  << std::endl;
+        return 1;
       } else {
         std::cerr << "glslc: error: invalid value '" << target_env_str
                   << "' in '--target-env=" << target_env_str << "'"
@@ -468,6 +464,29 @@ int main(int argc, char** argv) {
         return 1;
       }
       compiler.options().SetTargetEnvironment(target_env, version);
+    } else if (arg.starts_with("--target-spv=")) {
+      shaderc_spirv_version ver = shaderc_spirv_version_1_0;
+      const string_piece ver_str = arg.substr(std::strlen("--target-spv="));
+      if (ver_str == "spv1.0") {
+        ver = shaderc_spirv_version_1_0;
+      } else if (ver_str == "spv1.1") {
+        ver = shaderc_spirv_version_1_1;
+      } else if (ver_str == "spv1.2") {
+        ver = shaderc_spirv_version_1_2;
+      } else if (ver_str == "spv1.3") {
+        ver = shaderc_spirv_version_1_3;
+      } else if (ver_str == "spv1.4") {
+        ver = shaderc_spirv_version_1_4;
+      } else if (ver_str == "spv1.5") {
+        ver = shaderc_spirv_version_1_5;
+      } else if (ver_str == "spv1.6") {
+        ver = shaderc_spirv_version_1_6;
+      } else {
+        std::cerr << "glslc: error: invalid value '" << ver_str
+                  << "' in '--target-spv=" << ver_str << "'" << std::endl;
+        return 1;
+      }
+      compiler.options().SetTargetSpirv(ver);
     } else if (arg.starts_with("-mfmt=")) {
       const string_piece binary_output_format =
           arg.substr(std::strlen("-mfmt="));
@@ -480,6 +499,9 @@ int main(int argc, char** argv) {
       } else if (binary_output_format == "c") {
         compiler.SetSpirvBinaryOutputFormat(
             glslc::FileCompiler::SpirvBinaryEmissionFormat::CInitList);
+      } else if (binary_output_format == "wgsl") {
+        compiler.SetSpirvBinaryOutputFormat(
+            glslc::FileCompiler::SpirvBinaryEmissionFormat::WGSL);
       } else {
         std::cerr << "glslc: error: invalid value '" << binary_output_format
                   << "' in '-mfmt=" << binary_output_format << "'" << std::endl;
@@ -487,7 +509,7 @@ int main(int argc, char** argv) {
       }
     } else if (arg.starts_with("-x")) {
       string_piece option_arg;
-      if (!GetOptionArgument(argc, argv, &i, "-x", &option_arg)) {
+      if (!shaderc_util::GetOptionArgument(argc, argv, &i, "-x", &option_arg)) {
         std::cerr
             << "glslc: error: argument to '-x' is missing (expected 1 value)"
             << std::endl;
@@ -533,7 +555,8 @@ int main(int argc, char** argv) {
       }
     } else if (arg == "-MF") {
       string_piece dep_file_name;
-      if (!GetOptionArgument(argc, argv, &i, "-MF", &dep_file_name)) {
+      if (!shaderc_util::GetOptionArgument(argc, argv, &i, "-MF",
+                                           &dep_file_name)) {
         std::cerr
             << "glslc: error: missing dependency info filename after '-MF'"
             << std::endl;
@@ -543,7 +566,8 @@ int main(int argc, char** argv) {
           std::string(dep_file_name.data(), dep_file_name.size()));
     } else if (arg == "-MT") {
       string_piece dep_file_name;
-      if (!GetOptionArgument(argc, argv, &i, "-MT", &dep_file_name)) {
+      if (!shaderc_util::GetOptionArgument(argc, argv, &i, "-MT",
+                                           &dep_file_name)) {
         std::cerr << "glslc: error: missing dependency info target after '-MT'"
                   << std::endl;
         return 1;
@@ -589,7 +613,7 @@ int main(int argc, char** argv) {
       }
     } else if (arg.starts_with("-I")) {
       string_piece option_arg;
-      if (!GetOptionArgument(argc, argv, &i, "-I", &option_arg)) {
+      if (!shaderc_util::GetOptionArgument(argc, argv, &i, "-I", &option_arg)) {
         std::cerr
             << "glslc: error: argument to '-I' is missing (expected 1 value)"
             << std::endl;
